(*----------------------------------------------------------------------------*
 *  Direct3D Particle System 1 Demo                                           *
 *  This demo shows differences between various techiques (right and wrong)   *
 *  what generate particles from emitter in uniform directions.               *
 *  Sample uses vertex shader to calculate simple "phisics" of particles -    *
 *  linear raising and fading - in hardware.                                  *
 *  Sample shows graph of particle dencity in ares highlighted by rotating    *
 *  conem so user can see and determine by yourself what emitter is the best  *
 *  (actually "Shere" one is recommended)                                     *
 *                                                                            *
 *  Copyright (c) 2002 by Alexey Barkovoy                                     *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  Modified: 16-Dec-2002                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)

{$I DirectX.inc}

unit ParticleSystem1Unit;

interface

uses
  Windows, Messages,
  D3DX8, D3DApp, D3DFont, D3DFile, D3DUtil, DXUtil, {$I UseD3D8.inc};

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
const
  PARTICLE_COUNT = 10000;      // total on screen particle count
  FADE_TIME = 5.0;             // seconds
  PARTICLE_SIZE = 1;           // Set desired particle size
  PARTICLE_ScaleFactor = 1000; // Set scale factor (with distance to particle)

  ConeAngle = 30*Pi/180;       // Ange on scanning cone

  GraphSize = 200;             // Width of graph (in pixels/records)
  GraphHeight = 100;           // Height of graph in pixels

const
  ShaderName: PChar = 'ParticleSystem1.vsh';

type
  TGeneratorType = (gtPolar, gtCube, gtSphere, gtWrong);

//-----------------------------------------------------------------------------
// Name: struct <TMirrorVertex>
// Desc: Custom mirror vertex type
//-----------------------------------------------------------------------------
type
  PMirrorVertex = ^TMirrorVertex;
  TMirrorVertex = packed record
    p: TD3DXVector3;
    n: TD3DXVector3;
    color: DWORD;
  end;
  PAMirrorVertex = ^AMirrorVertex;
  AMirrorVertex = array[0..4] of TMirrorVertex;

const
  D3DFVF_MIRRORVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_DIFFUSE);


//-----------------------------------------------------------------------------
// Name: struct <TParticleVertex>
// Desc: Custom vertex type used for particle rendering
//-----------------------------------------------------------------------------
type
  // Custom D3D vertex format used by the vertex buffer
  PParticleVertex = ^TParticleVertex;
  TParticleVertex = packed record
    dir: TD3DXVector3;  // direction of particle motion
    col: TD3DColor;
    emitTime: Single;   // timestamp of initial particle emitment
  end;

  PParticleVertexArray = ^TParticleVertexArray;
  TParticleVertexArray = array [0..MaxInt div SizeOf(TParticleVertex) - 1] of TParticleVertex;

const
  D3DFVF_MYVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL);

type
//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the
//       generic functionality needed in all Direct3D samples. CMyD3DApplication
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    m_Generator: TGeneratorType; // What Unidirectional generator we use?

    m_pFont: CD3DFont;           // Font for drawing text
    m_pFontSmall: CD3DFont;      // Font for drawing hellp text
    m_pSphereMesh: ID3DXMesh;    // Representation of Sphere
    m_pConeMesh: ID3DXMesh;      // Representation of cone of statistical interest
    m_matCone: TD3DXMatrix;      // Cone's local matrix

    m_pFlare: IDirect3DTexture8; // Flare texture

    m_pMirrorVB: IDirect3DVertexBuffer8;

    m_vEyePt: TD3DXVector3;      // Vectors defining the camera
    m_vLookatPt: TD3DXVector3;
    m_vUpVec: TD3DXVector3;
    m_matProj: TD3DXMatrix;

    m_Particles: PParticleVertexArray;          // CPU-side particles
    m_ParticlesVB: IDirect3DVertexBuffer8;      // GPU-side particles
    m_ParticlesVS: DWORD;                       // Handle to vertex shader

    m_GraphData: array[0..GraphSize-1] of Single;

    fShowHelp: Boolean;
    fPaused: Boolean;
    fRotateAroundY: Boolean;

    function RenderMirror: HRESULT;
    function RenderScene: HRESULT;
    function RenderGraph: HRESULT;

    procedure UpdateParticleSystem(OnInit: Boolean{$IFDEF SUPPORTS_DEFAULTPARAMS} = False{$ENDIF});
  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function ConfirmDevice(var pCaps: TD3DCaps8; dwBehavior: DWORD; Format: TD3DFormat): HRESULT; override;
    function FinalCleanup: HRESULT; override;
  public
    constructor Create;
    function MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;

implementation

uses Math;

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited Create(0);

  Randomize;
  m_Generator:= gtWrong; //gtPolar;
  fShowHelp:= False;
  fPaused:= False;
  fRotateAroundY:= True;

  m_strWindowTitle    := 'Particle System 1: Unidirectional particle generator';
  m_bUseDepthBuffer   := True; // need Z-Buffer for stencil rendering
  m_dwMinDepthBits    := 16;
  m_dwMinStencilBits  := 4;

  m_pFont             := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
  m_pFontSmall        := CD3DFont.Create('Arial', 10, D3DFONT_BOLD);
  m_pMirrorVB         := nil;
  m_ParticlesVB       := nil;
  m_pSphereMesh       := nil;
  m_pConeMesh         := nil;
end;

procedure CMyD3DApplication.UpdateParticleSystem(OnInit: Boolean{$IFDEF SUPPORTS_DEFAULTPARAMS} = False{$ENDIF});
  // ========================================================
  // ---=== This is main function of sample            ===---
  // ========================================================
  procedure GenerateNewVector(var v: TD3DXVector3);
  var
    u, vv, azimuthAngle, polarAngle: Single;
  begin
    case m_Generator of
      gtPolar:
      begin
        // u, v = random numbers between 0 and 1 inclusive
        u:= Random; vv:= Random;

        azimuthAngle := 2 * PI * u;
        polarAngle := ArcCos( 2 * vv - 1 );
        v.x := cos(azimuthAngle) * sin(polarAngle);
        v.y := sin(azimuthAngle) * sin(polarAngle);
        v.z := cos(polarAngle);
      end;

      gtCube:
      begin
        v.x:= Random*2 - 1;
        v.y:= Random*2 - 1;
        v.z:= Random*2 - 1;
      end;

      gtSphere:
      begin
        repeat // until we get point inside sphere
          v.x:= Random*2 - 1;
          v.y:= Random*2 - 1;
          v.z:= Random*2 - 1;
        until D3DXVec3LengthSq(v) <= 1;
      end;

      gtWrong:
      begin // Same as cube as gtCube but only fill one half
        v.x:= Random;
        v.y:= Random*2 - 1;
        v.z:= Random*2 - 1;

        v.z:= Random;
        v.x:= Random*2 - 1;
        v.y:= Random*2 - 1;

      end;
    end;
    D3DXVec3Normalize(v, v);
  end;
  // ========================================================
  // ========================================================
var
  i, n: Integer;
  m: TD3DXMatrix;
  outV: TD3DXVector4;
begin
  if OnInit then
  begin
    for i:= 0 to PARTICLE_COUNT - 1 do
    begin
      GenerateNewVector(m_Particles^[i].dir);
      m_Particles^[i].col:= D3DCOLOR_COLORVALUE(1, 1, 1, 1);
      m_Particles^[i].emitTime:= m_fTime - Random * FADE_TIME;
    end;
  end else
  begin
    for i:= 0 to PARTICLE_COUNT - 1 do
    begin
      if ((m_fTime - m_Particles^[i].emitTime) > FADE_TIME) {and (Random > 0.3)} then
      begin
        GenerateNewVector(m_Particles^[i].dir);
        m_Particles^[i].col:= D3DCOLOR_COLORVALUE(0, 1, 0, 1);
        m_Particles^[i].emitTime:= m_Particles^[i].emitTime + FADE_TIME;
      end;
    end;

    // Recalculate how many particles fit in selected sphere slice
    Move(m_GraphData[0], m_GraphData[1], SizeOf(m_GraphData) - SizeOf(m_GraphData[0]));
    n:= 0;
    D3DXMatrixInverse(m, nil, m_matCone);
    for i:= 0 to PARTICLE_COUNT - 1 do
    begin
      D3DXVec3Transform(outV, m_Particles^[i].dir, m);
      if outV.z > 1.0{Radius}*1.0{Raduis}*Cos(ConeAngle) then Inc(n);
    end;
    m_GraphData[0]:= (n / PARTICLE_COUNT) / ((1-Cos(ConeAngle))/2);
  end;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  // Initialize the camera's orientation
  m_vEyePt    := D3DXVector3(0.0, 2.0, -6.5);
  m_vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
  m_vUpVec    := D3DXVector3(0.0, 1.0,  0.0);

  GetMem(m_Particles, SizeOf(TParticleVertex)*PARTICLE_COUNT);
  UpdateParticleSystem(True);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  v: PAMirrorVertex;
begin
  // Initialize the font's internal textures
  Result:= m_pFont.InitDeviceObjects(m_pd3dDevice);
  if Failed(Result) then Exit;

  Result:= m_pFontSmall.InitDeviceObjects(m_pd3dDevice);
  if Failed(Result) then Exit;

  // Set up the geometry objects
  if FAILED(D3DUtil_CreateTexture(m_pd3dDevice, PChar('flare2.bmp'), m_pFlare{$IFNDEF SUPPORTS_DEFAULTPARAMS}, D3DFMT_UNKNOWN{$ENDIF})) then
  begin
    Result:= D3DAPPERR_MEDIANOTFOUND;
    Exit;
  end;

  // Create a square for rendering the mirror
  if FAILED(m_pd3dDevice.CreateVertexBuffer(4*SizeOf(TMirrorVertex),
       D3DUSAGE_WRITEONLY, D3DFVF_MIRRORVERTEX, D3DPOOL_MANAGED, m_pMirrorVB)) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  // Initialize the mirror's vertices
  m_pMirrorVB.Lock(0, 0, PBYTE(v), 0);
  v^[0].p := D3DXVector3(-1.5, 1.5, 3.0);
  v^[2].p := D3DXVector3(-1.5,-1.5, 3.0);
  v^[1].p := D3DXVector3( 1.5, 1.5, 3.0);
  v^[3].p := D3DXVector3( 1.5,-1.5, 3.0);
  v^[0].n := D3DXVector3(0.0,0.0,-1.0);
  v^[1].n := v^[0].n;
  v^[2].n := v^[0].n;
  v^[3].n := v^[0].n;
  v^[0].color := $80ffffff;
  v^[1].color := $80ffffff;
  v^[2].color := $80ffffff;
  v^[3].color := $80ffffff;
  m_pMirrorVB.Unlock;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  matWorld, matView: TD3DXMatrix;
  fAspect: Single;
  light: TD3DLight8;
  dwParticlesVertexDecl: array[0..4] of DWORD;
begin
  m_pFont.RestoreDeviceObjects;
  m_pFontSmall.RestoreDeviceObjects;

  // Create vertex shader for point sprite system
  dwParticlesVertexDecl[0]:= D3DVSD_STREAM(0);
  dwParticlesVertexDecl[1]:= D3DVSD_REG(D3DVSDE_POSITION, D3DVSDT_FLOAT3);   // Position
  dwParticlesVertexDecl[2]:= D3DVSD_REG(D3DVSDE_DIFFUSE,  D3DVSDT_D3DCOLOR); // Color
  dwParticlesVertexDecl[3]:= D3DVSD_REG(6,                D3DVSDT_FLOAT1);   // Point born-time
  dwParticlesVertexDecl[4]:= D3DVSD_END;

  Result := D3DUtil_CreateVertexShader(m_pd3dDevice, ShaderName,
                                       @dwParticlesVertexDecl,
                                       m_ParticlesVS);
  if FAILED(Result) then Exit;

  // Create a VB for rendering of the particle system
  if FAILED(m_pd3dDevice.CreateVertexBuffer(SizeOf(TParticleVertex)*PARTICLE_COUNT,
       D3DUSAGE_WRITEONLY or D3DUSAGE_DYNAMIC, 0 {non-FVF VB},
       D3DPOOL_DEFAULT, m_ParticlesVB)) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  if FAILED(D3DXCreateCylinder(m_pd3dDevice, 0.0,
              1.0{Radius}*Sin(ConeAngle), 1.0{Raduis}*Cos(ConeAngle), 200, 20, m_pConeMesh, nil))
  then Exit;

  if FAILED(D3DXCreateSphere(m_pd3dDevice, 1.0, 40, 40, m_pSphereMesh, nil))
  then Exit;


  // Set up the textures
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);

  m_pd3dDevice.SetTextureStageState(1, D3DTSS_COLOROP,   D3DTOP_DISABLE);
  m_pd3dDevice.SetTextureStageState(1, D3DTSS_ALPHAOP,   D3DTOP_DISABLE);

  // Set miscellaneous render states
  m_pd3dDevice.SetRenderState(D3DRS_DITHERENABLE,   iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_SPECULARENABLE, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE,        iFalse);
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING,       iTrue);

  // Set initial alpha-blending mode
  m_pd3dDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

  // Set up the matrices
  D3DXMatrixIdentity(matWorld);
  D3DXMatrixLookAtLH(matView, m_vEyePt, m_vLookatPt, m_vUpVec);
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  D3DXMatrixPerspectiveFovLH(m_matProj, D3DX_PI/4, fAspect, 1.0, 100.0);
  m_pd3dDevice.SetTransform(D3DTS_WORLD,      matWorld);
  m_pd3dDevice.SetTransform(D3DTS_VIEW,       matView);
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, m_matProj);

  // Set up a light
  if (m_d3dCaps.VertexProcessingCaps and D3DVTXPCAPS_DIRECTIONALLIGHTS = D3DVTXPCAPS_DIRECTIONALLIGHTS) or
     (m_dwCreateFlags and D3DCREATE_HARDWARE_VERTEXPROCESSING = 0) then
  begin
    D3DUtil_InitLight(light, D3DLIGHT_DIRECTIONAL, 0.2, -1.0, -0.2);
    m_pd3dDevice.SetLight(0, light);
    m_pd3dDevice.LightEnable(0, True);
  end;
  m_pd3dDevice.SetRenderState(D3DRS_AMBIENT, $ff555555);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove;
var
  quat: TD3DXQuaternion;
  matView: TD3DXMatrix;
  pData: PByte;
begin
  // Set the teapot's local matrix (rotating about the y-axis)
  if not fPaused then
    if fRotateAroundY
    then D3DXMatrixRotationY(m_matCone, m_fTime)
    else D3DXMatrixRotationX(m_matCone, m_fTime);

  // When the window has focus, let the mouse adjust the camera view
  if (GetFocus <> 0) then
  begin
    quat := D3DUtil_GetRotationFromCursor(m_hWnd{$IFNDEF SUPPORTS_DEFAULTPARAMS}, 1.0{$ENDIF});
    m_vEyePt.x := 50*quat.y;
    m_vEyePt.y := 50*quat.x;
    m_vEyePt.z := 4 - Sqrt(50.0 - 25*quat.x*quat.x - 25*quat.y*quat.y);

    D3DXMatrixLookAtLH(matView, m_vEyePt, m_vLookatPt, m_vUpVec);
    m_pd3dDevice.SetTransform(D3DTS_VIEW, matView);
  end;

  UpdateParticleSystem{$IFNDEF SUPPORTS_DEFAULTPARAMS}(False){$ENDIF};

  m_ParticlesVB.Lock(0, 0, pData, D3DLOCK_DISCARD);
  Move(m_Particles^[0], pData^, SizeOf(TParticleVertex)*PARTICLE_COUNT);
  m_ParticlesVB.Unlock;

  Result:= S_OK;
end;

function FtoDW(f: Single): DWORD;
begin
  Result:= PDWord(@f)^;
end;

//-----------------------------------------------------------------------------
// Name: RenderScene()
// Desc: Renders all objects in the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderScene: HRESULT;
 procedure SetupVShaderConstants;
 var
   matWorldSaved, matView, matWorldView, matTotal: TD3DXMatrix;
   fConst4: TD3DXVector4;
 begin
    m_pd3dDevice.GetTransform(D3DTS_WORLD, matWorldSaved);
   // Setup Matrices
   D3DXMatrixLookAtLH(matView, m_vEyePt, m_vLookatPt, m_vUpVec);
   D3DXMatrixMultiply(matWorldView, matWorldSaved, matView);
   D3DXMatrixMultiply(matTotal, matWorldView, m_matProj);

   D3DXMatrixTranspose(matWorldView, matWorldView);
   D3DXMatrixTranspose(matTotal, matTotal);

   m_pd3dDevice.SetVertexShaderConstant(0, matTotal, 4); // Total Matrix
   m_pd3dDevice.SetVertexShaderConstant(4, matWorldView, 4); // WorldView Matrix

   // Setup (Time, FadeFactor, 0, 1)
   fConst4:= D3DXVector4(m_fTime, 2 / FADE_TIME, 0, 1);
   m_pd3dDevice.SetVertexShaderConstant(8, fConst4, 1); // (Time, FadeFactor, 0, 1)

   fConst4:= D3DXVector4(m_d3dpp.BackBufferHeight * PARTICLE_SIZE, PARTICLE_ScaleFactor, 0, 1);
   m_pd3dDevice.SetVertexShaderConstant(9, fConst4, 1);
 end;
var
  matLocal, matWorldSaved, m1: TD3DXMatrix;
  mat: TD3DMaterial8;
begin
  m_pd3dDevice.GetTransform(D3DTS_WORLD, matWorldSaved);

// ****** Render POINT SPRITES ******

  SetupVShaderConstants; // Setup VertexShader constants for particle system
  m_pd3dDevice.SetStreamSource(0, m_ParticlesVB, SizeOf(TParticleVertex));
  m_pd3dDevice.SetVertexShader(m_ParticlesVS);

  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG2);

  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);

  D3DUtil_InitMaterial(mat, 0.0, 1.0, 0.0, 0.5);
  m_pd3dDevice.SetMaterial(mat);

  m_pd3dDevice.SetTexture(0, m_pFlare);
  m_pd3dDevice.SetRenderState(D3DRS_POINTSPRITEENABLE, iTrue);
  m_pd3dDevice.DrawPrimitive(D3DPT_POINTLIST, 0, PARTICLE_COUNT);
  m_pd3dDevice.SetRenderState(D3DRS_POINTSPRITEENABLE, iFalse);
  m_pd3dDevice.SetTexture(0, nil);


// ****** Render CONE ******

  D3DXMatrixTranslation(m1, 0, 0, 1.0{Raduis}*Cos(ConeAngle)/2);
  D3DXMatrixMultiply(matLocal, m_matCone, matWorldSaved);
  D3DXMatrixMultiply(matLocal, m1, matLocal);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, matLocal);

  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

  D3DUtil_InitMaterial(mat, 10.0, 1.0, 0.0, 0.1);
  m_pd3dDevice.SetMaterial(mat);

  m_pConeMesh.DrawSubset(0);

  
// ****** Render SPHERE ******

  m_pd3dDevice.SetTransform(D3DTS_WORLD, matWorldSaved);

  D3DUtil_InitMaterial(mat, 0.5, 0.5, 0.0, 0.1);
  m_pd3dDevice.SetMaterial(mat);

  m_pSphereMesh.DrawSubset(0);

  D3DUtil_InitMaterial(mat, 0.0, 1.0, 0.0, 0.5);
  m_pd3dDevice.SetMaterial(mat);


  // Restore the modified render states
  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);

  m_pd3dDevice.SetTransform(D3DTS_WORLD, matWorldSaved);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: RenderMirror()
// Desc: Renders the scene as reflected in a mirror. The corners of the mirror
//       define a plane, which is used to build the reflection matrix. The
//       scene is rendered with the cull-mode reversed, since all normals in
//       the scene are likewise reflected.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderMirror: HRESULT;
var
  matWorldSaved: TD3DXMatrix;
  matReflectInMirror: TD3DXMatrix;
  plane: TD3DXPlane;
  a, b, c: TD3DXVector3;
begin
  // Render mirror in stencil buffer (and not rendering it to frame buffer)
  // (StencilRef & StencilMask) CompFunc (StencilBufferValue & StencilMask)
  begin
    // Turn depth buffer off, and stencil buffer on
    m_pd3dDevice.SetRenderState(D3DRS_STENCILENABLE, iTrue);

    m_pd3dDevice.SetRenderState(D3DRS_STENCILFUNC,     D3DCMP_ALWAYS);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILREF,      $1);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILMASK,     $ffffffff);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILWRITEMASK,$ffffffff);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILZFAIL, D3DSTENCILOP_KEEP);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILFAIL,  D3DSTENCILOP_KEEP);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILPASS,  D3DSTENCILOP_REPLACE);

    // We are filling stencil buffer with mirror area
    m_pd3dDevice.SetStreamSource(0, m_pMirrorVB, SizeOf(TMirrorVertex));
    m_pd3dDevice.SetVertexShader(D3DFVF_MIRRORVERTEX);

    m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ZERO);
    m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);

    {-->}m_pd3dDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);{<--}

    m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
    m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

    // turn TESTING mode of stencil instead of WRITING
    m_pd3dDevice.SetRenderState(D3DRS_STENCILFUNC,  D3DCMP_EQUAL);
    m_pd3dDevice.SetRenderState(D3DRS_STENCILPASS,  D3DSTENCILOP_KEEP);
  end;

  // Save the world matrix so it can be restored
  m_pd3dDevice.GetTransform(D3DTS_WORLD, matWorldSaved);

  // Get the four corners of the mirror. (This should be dynamic rather than
  // hardcoded.)
  a:= D3DXVector3(-1.5, 1.5, 3.0);
  b:= D3DXVector3( 1.5, 1.5, 3.0);
  c:= D3DXVector3(-1.5,-1.5, 3.0);

  // Construct the reflection matrix
  D3DXPlaneFromPoints(plane, a, b, c);
  D3DXMatrixReflect(matReflectInMirror, plane);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, matReflectInMirror);

  // Reverse the cull mode (since normals will be reflected)
  m_pd3dDevice.SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);

  // Render the scene
  RenderScene;

  // Restore the modified render states
  m_pd3dDevice.SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, matWorldSaved);
  m_pd3dDevice.SetRenderState(D3DRS_STENCILENABLE, iFalse); // Turn off stencil operations

  // Finally, render the mirror itself (as an alpha-blended quad)
  m_pd3dDevice.SetStreamSource(0, m_pMirrorVB, SizeOf(TMirrorVertex));
  m_pd3dDevice.SetVertexShader(D3DFVF_MIRRORVERTEX);
  m_pd3dDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);

  Result:= S_OK;
end;

function CMyD3DApplication.RenderGraph: HRESULT;
type
  TData = packed record
    vec: TD3DXVector4;
    col: TD3DColor;
  end;
var
  Data: array[0..GraphSize-1 + 4 + 2] of TData;
  // 4 - for Rect;
  // 2 - for Center Line
  // GraphSize - for data

  x, y, t: Single;
  i: Integer;
const
  D3DFVF_GRAPH = (D3DFVF_XYZRHW or D3DFVF_DIFFUSE);
begin
  Result:= S_OK;

  // Prepare data
  x:= m_d3dpp.BackBufferWidth - GraphSize - 10;
  y:= m_d3dpp.BackBufferHeight - GraphHeight - 10 ;

  Data[0].vec:= D3DXVector4(x            , y              , 0.9, 1);
  Data[1].vec:= D3DXVector4(x + GraphSize, y              , 0.9, 1);
  Data[2].vec:= D3DXVector4(x + GraphSize, y + GraphHeight, 0.9, 1);
  Data[3].vec:= D3DXVector4(x            , y + GraphHeight, 0.9, 1);
  Data[0].col:= D3DCOLOR_RGBA(0, 155, 0, 12);
  Data[1].col:= D3DCOLOR_RGBA(0, 155, 0, 12);
  Data[2].col:= D3DCOLOR_RGBA(0, 155, 0, 12);
  Data[3].col:= D3DCOLOR_RGBA(0, 155, 0, 12);

  Data[4].vec:= D3DXVector4(x            , y + GraphHeight*3/4, 0.9, 1);
  Data[5].vec:= D3DXVector4(x + GraphSize, y + GraphHeight*3/4, 0.9, 1);
  Data[4].col:= D3DCOLOR_RGBA(255, 0, 0, 200);
  Data[5].col:= D3DCOLOR_RGBA(255, 0, 0, 200);

  for i:= 0 to GraphSize - 1 do
  begin
    t:= m_GraphData[i];
    if t > 4 then t:= 4;
    Data[6+i].vec:= D3DXVector4(x+i, y + (GraphHeight - t*GraphHeight/4), 0.9, 1);
    Data[6+i].col:= D3DCOLOR_RGBA(0, 255, 0, 200);
  end;

  // Render it all
  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  m_pd3dDevice.SetVertexShader(D3DFVF_GRAPH);

  m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, Data, SizeOf(TData));
  m_pd3dDevice.DrawPrimitiveUP(D3DPT_LINELIST, 1, Data[4], SizeOf(TData));
  m_pd3dDevice.DrawPrimitiveUP(D3DPT_LINESTRIP, GraphSize-1, Data[6], SizeOf(TData));

  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
end;

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER or D3DCLEAR_STENCIL,
                     $000000ff, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Render the scene
    RenderScene;

    // Render the scene in the mirror
    RenderMirror;

    // Render graph
    RenderGraph;

    // Output statistics
    m_pFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
    m_pFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);

    if fShowHelp then
    begin
      m_pFontSmall.DrawText(2, 40, D3DCOLOR_ARGB(255,250,200,100),
                            'Keyboard controls:' );
      m_pFontSmall.DrawText(57, 60, D3DCOLOR_ARGB(255,250,200,100),
                            'Particle generator: Polar/Cube/Sphere/"Wrong"'#10 +
                            'Change cone rotation axis'#10 +
                            'Pause/Resume rotation'#10 +
                            'Help'#10'Change device'#10'Exit');

      m_pFontSmall.DrawText(50, 60, D3DCOLOR_ARGB(200,255,255,100),
                            '-'#10'-'#10'-'#10'-'#10'-'#10'-');

      // Draw controls with high contrast
      m_pFontSmall.DrawText(6, 61, D3DCOLOR_ARGB(255,0,0,0),
                            '1/2/3/4'#10 +
                            'R'#10 +
                            'P'#10 +
                            'F1'#10'F2'#10'Esc');
      m_pFontSmall.DrawText(5, 60, D3DCOLOR_ARGB(255,255,255,255),
                            '1/2/3/4'#10 +
                            'R'#10 +
                            'P'#10 +
                            'F1'#10'F2'#10'Esc');
    end else
    begin
      m_pFontSmall.DrawText(2, 40, D3DCOLOR_ARGB(255,200,200,100),
                            'Press F1 for help' );
    end;

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects;
begin
  m_pFont.InvalidateDeviceObjects;
  m_pFontSmall.InvalidateDeviceObjects;
  SAFE_RELEASE(m_pConeMesh);
  SAFE_RELEASE(m_pSphereMesh);

  if (m_ParticlesVS <> 0) then
  begin
    m_pd3dDevice.DeleteVertexShader(m_ParticlesVS);
    m_ParticlesVS:= 0;
  end;

  SAFE_RELEASE(m_ParticlesVB);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects;
begin
  m_pFont.DeleteDeviceObjects;
  m_pFontSmall.DeleteDeviceObjects;

  SAFE_RELEASE(m_pMirrorVB);
  SAFE_RELEASE(m_pFlare);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);
  SAFE_DELETE(m_pFontSmall);

  FreeMem(m_Particles);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device intialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(var pCaps: TD3DCaps8; dwBehavior: DWORD;
  Format: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  if (dwBehavior and D3DCREATE_PUREDEVICE) = D3DCREATE_PUREDEVICE
  then Exit; // GetTransform doesn't work on PUREDEVICE

  if (dwBehavior and D3DCREATE_HARDWARE_VERTEXPROCESSING = D3DCREATE_HARDWARE_VERTEXPROCESSING) or
     (dwBehavior and D3DCREATE_MIXED_VERTEXPROCESSING = D3DCREATE_MIXED_VERTEXPROCESSING) then
  begin
    if (pCaps.MaxPointSize <= 1.0) then Exit;

    // Verify supported vertex shader version
    if (D3DSHADER_VERSION_MAJOR(pCaps.VertexShaderVersion) < 1) then Exit;
  end;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle key and menu input
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM;
  lParam: LPARAM): LRESULT;
begin
  // Trap context menu
  if (WM_CONTEXTMENU = uMsg) then
  begin
    Result := 0;
    Exit;
  end;

  // Handle key presses
  if (WM_KEYDOWN = uMsg) then
  begin
    if wParam in [Ord('1')..Ord('4')] then
    begin
      m_Generator:= TGeneratorType(wParam - Ord('1'));
    end;

    if wParam = Ord('R') then fRotateAroundY:= not fRotateAroundY;
    if wParam = Ord('P') then fPaused:= not fPaused;
    if wParam = VK_F1    then fShowHelp:= not fShowHelp;
  end;

  // Pass remaining messages to default handler
  Result := inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

end.

